/*
   This file is part of the Aulib library.
   Name: aucut.cpp
   Copyright (C) 2004 Silvia Pfeiffer

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: pfeiffer@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

#include "aucut.h"

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
//                CutDetectorUsingFFT 

void CutDetectorUsingFFT::deviations(Float eq, Float sim) {
  if (eq > sim)
    throw Exception("equality deviation must be less then similarity deviation.");
  _eqDev = eq;
  _simDev = sim;
}

////////////////////////////////////////////////////////////

FFTData* CutDetectorUsingFFT::analyse(const AudioCache& aucache) {

  // be sure that _tempSpec has the necessary size
  if (_tempSpec.size() != _fftTool->winSize())
    _tempSpec.resize(_fftTool->winSize());

  // if winSize is changed - delete history
  if (_history.size() != _fftTool->winSize())
    reset();

  _fftTool->fft(aucache, &_tempSpec);
  return &_tempSpec;
}

////////////////////////////////////////////////////////////

CutDetectorUsingFFT::Similarity CutDetectorUsingFFT::compareTo(const FFTData& ind) {
  const Float dev = euclVecDist(_history, ind);
#ifdef TEST
  cres << "Deviation:\t" << dev << endl;
#endif
  if (dev < _eqDev)
    return EQUAL;
  if (dev < _simDev)
    return SIMILAR;
  return UNEQUAL;
}

Float CutDetectorUsingFFT::euclVecDist(const FFTData& spec1, const FFTData& spec2) const {

  Float dist = 0.0;

  for (size_t i = 0; i < spec1.size() && i < spec2.size(); i++)
    dist += pow((spec1.*_convFunc)(i) - (spec2.*_convFunc)(i), 2);
  dist = sqrt(dist);

  return dist;
}

////////////////////////////////////////////////////////////

void CutDetectorUsingFFT::adjust(const FFTData& spec, bool cut) {

  if (!cut) {
    for (size_t i = 0; i < _history.size(); i++)
      _history[i] = _weight * _history[i] + (1 - _weight) * spec[i];
  }
  else {
    _history = spec;
  }
}

