/*
   This file is part of the Aulib library.
   Name: auformat.h
   Copyright (C) 2004 Silvia Pfeiffer

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: pfeiffer@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

#ifndef _AU_FORMAT_H_
#define _AU_FORMAT_H_

#include "autypes.h"

#ifdef AF
#include <AF/AFlib.h>
#include <AF/AFUtils.h>
#endif

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
//                    DataFormat

class DataFormat {
  public:
    enum FormatCode {
      mulaw   = 1, // Log PCM rep. 8 bits per sample
      lin8    = 2, // Linear PCM rep. 8 bits per sample
      lin16   = 3, // Linear PCM rep. 16 bits per sample
      lin24   = 4, // Linear PCM rep. 24 bits per sample
      lin32   = 5, // Linear PCM rep. 32 bits per sample
      IEEEs   = 6, // IEEE single prec. -1.0 <= x < 1.0 
                   // Floating point format             
      IEEEd   = 7, // IEEE double prec. -1.0 <= x < 1.0 
                   // Floating point format             
      FRAG    = 8, // Fragmented sampled data           
      UNKNOWN = 9  // whatever                          
    };

    static const char* name(FormatCode);
    static uint32 bytesInSample(FormatCode);
#ifdef AF
    static AEncodeType AFEncodeType(FormatCode);
#endif

  private:
    static const char* _names[];
    static const uint32 _bytesInSample[];
};

////////////////////////////////////////////////////////////

inline
const char* DataFormat::name(FormatCode type) {
  return _names[type - 1];
}

////////////////////////////////////////////////////////////

inline
uint32 DataFormat::bytesInSample(FormatCode type) {
  return _bytesInSample[type - 1];
}

#endif

