/*
   This file is part of the Aulib library.
   Name: convert.cpp
   Copyright (C) 2004 Silvia Pfeiffer

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: pfeiffer@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

#include "convert.h"

const string AMP_MODE_MAX = "max";
const string AMP_MODE_MIN = "min";
const string AMP_MODE_AVE = "ave";
const string AMP_MODE_ABS = "abs";

AuAmplitude::Mode atoAmpMode(char* argv) {
  if (argv == AMP_MODE_MAX) {
		return AuAmplitude::MAX;
	} else if (argv == AMP_MODE_MIN) {
		return AuAmplitude::MIN;
	} else if (argv == AMP_MODE_AVE) {
		return AuAmplitude::AVE;
	} else if (argv == AMP_MODE_ABS) {
		return AuAmplitude::ABS;
	} else
		return AuAmplitude::MAX;
}

const string SEEK_MODE_BEG = "b";
const string SEEK_MODE_CUR = "c";
const string SEEK_MODE_END = "e";

AudioData::SeekMode atoSeekMode(char* argv) {
  if (argv == SEEK_MODE_BEG) {
		return AudioData::BEG;
	} else if (argv == SEEK_MODE_CUR) {
		return AudioData::CUR;
	} else if (argv == SEEK_MODE_END) {
		return AudioData::END;
	} else
		return AudioData::BEG;
}

const string LENGTH_FORMAT_SECS = "s";
const string LENGTH_FORMAT_FRAMES = "f";
const string LENGTH_FORMAT_BYTES = "b";
const string LENGTH_FORMAT_SAMPLES = "a";

AuLength::Format atoLengthFormat(char* argv) {
  if (argv == LENGTH_FORMAT_SECS) {
		return AuLength::SECONDS;
  } else if (argv == LENGTH_FORMAT_FRAMES) {
		return AuLength::FRAMES;
  } else if (argv == LENGTH_FORMAT_BYTES) {
		return AuLength::BYTES;
  } else if (argv == LENGTH_FORMAT_SAMPLES) {
		return AuLength::SAMPLES;
	} else
		return AuLength::SECONDS;
}

const string WIN_FUNC_HAMMING = "HAMMING";
const string WIN_FUNC_HANNING = "HANNING";
const string WIN_FUNC_NONE = "NONE";

FFT::WinFunc atoWinFunc(char* argv) {
  if (argv == WIN_FUNC_HAMMING) {
		return FFT::HAMMING;
	} else if (argv == WIN_FUNC_HANNING) {
		return FFT::HANNING;
	} else if (argv == WIN_FUNC_NONE) {
		return FFT::NONE;
	} else
		return FFT::HAMMING;
}

const string CONV_METHOD_REAL = "M_REAL";
const string CONV_METHOD_AMP = "M_AMP";
const string CONV_METHOD_SQR = "M_SQR";
const string CONV_METHOD_DB = "M_DB";
const string CONV_METHOD_PHASE = "M_PHA";
const string CONV_METHOD_ADD = "M_ADD";

FFTData::ConvFunc atoConvFunc(char* argv) {
  if (argv == CONV_METHOD_REAL) {
		return &FFTData::real;
	} else
  if (argv == CONV_METHOD_AMP) {
		return &FFTData::amp;
	} else
  if (argv == CONV_METHOD_SQR) {
		return &FFTData::sqr;
	} else
  if (argv == CONV_METHOD_DB) {
		return &FFTData::dB;
	} else
  if (argv == CONV_METHOD_PHASE) {
		return &FFTData::phase;
	} else
  if (argv == CONV_METHOD_ADD) {
		return &FFTData::add;
	} else
		return &FFTData::amp;
}

