/*
   This file is part of the Aulib library.
   Name: scalarType.cpp
   Copyright (C) 2004 Silvia Pfeiffer

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: pfeiffer@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

#include <limits.h>
#include <math.h>
#include <float.h>

#include "byteOrder.h"
#include "scalarType.H"


scalarType::scalarType() :
  value(0), calculated(0) {
}

scalarType::scalarType(const scalarType& _scl) {
  calculated = _scl.calculated;
  value = _scl.value;
}

scalarType::scalarType(float val) :
  value(val)
{
}


scalarType&
scalarType::operator= (const scalarType& _scl) {
  if (this == &_scl) return *this;
  calculated = _scl.calculated;
  value = _scl.value;
  return *this;
}

float
scalarType::distance_L1 (const scalarType& s, float threshold) {
  float dist = 0;
  if ( (value>=threshold) || (s.value>=threshold) )
    dist = (float) fabs(value - s.value);
  return dist;
}


float
scalarType::distance_L2 (const scalarType& s, float threshold) {
  float dist = 0, diff =0;
  if ( (value>=threshold) || (s.value>=threshold) ) {
    diff = value - s.value;
    dist = (diff * diff);
  }
  return sqrt(dist);
}


void
scalarType::write(FILE *file) {
  fwrite (&calculated, sizeof(unsigned char), 1, file);

  float temp_value = value;
#ifdef DAMN_INTEL_BYTE_ORDER
  swap_int ((char*)&temp_value);
  fwrite ((char*)&temp_value, sizeof(unsigned char), 4, file);
#else
  fwrite (&temp_value, sizeof(float), 1, file);

#endif
}

void
scalarType::read(FILE *file) {
  fread(&calculated, sizeof(unsigned char), 1, file);
#ifdef DAMN_INTEL_BYTE_ORDER
  fread((char*)&value, sizeof(unsigned char), 4, file);
  swap_int ((char*)&value);
#else
  fread(&value, sizeof(float), 1, file);
#endif
}


ostream&
operator<<(ostream& os, const scalarType& s) {
  // print value
  os << s.value;
  os << endl;
  return os;
}
