/*
   This file is part of the Aulib library.
   Name: vectorType.H
   Copyright (C) 2004 Silvia Pfeiffer

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: pfeiffer@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

#ifndef _VECTORTYPE_H
#define _VECTORTYPE_H

#include <stdio.h>
#include <iostream.h>
#include <climits>

#ifndef RETURNVECTOR
#define RETURNVECTOR(vec) sprintf(buffer,"%p", &vec);
#endif
#ifndef RETURNVECTORPTR
#define RETURNVECTORPTR(vec) sprintf(buffer,"%p", vec);
#endif

struct VectorType {
  friend ostream& operator<<(ostream&, const VectorType&);
  //  friend istream& operator>>(istream&, VectorType&);

  int size;
  unsigned char calculated;
  float *values;

  VectorType(int _size);
  VectorType(const VectorType& _vec);
  VectorType(float val1, float val2);
  ~VectorType() { if (values) delete[] values; }

  VectorType& operator= (const VectorType& _vec);
  float& operator[] (int i) { return values[i]; }

  float distance_L1 (const VectorType& v, float threshold = 0);
  float distance_L2 (const VectorType& v, float threshold = 0);
  float distance_CCV (const VectorType& v, float threshold = 0);

  float max();
  float min();


  void read(FILE *file);
  void write(FILE *file);

  // returns the size of one vector in the DB
  size_t fileSize() {return (sizeof(float) * size + sizeof(unsigned char));}
};


inline VectorType&
GetVector(char* string)
{
  VectorType* vecPtr;
  sscanf(string,"%p",&vecPtr);
  return *vecPtr;
}


ostream&
operator<<(ostream& os, const VectorType& v);

#endif _VECTORTYPE_H
