/*
   This file is part of the Aulib library.
   Name: vectorType.cpp
   Copyright (C) 2004 Silvia Pfeiffer

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: pfeiffer@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

#include <climits>
#include <cmath>
#include <cfloat>

#include "byteOrder.h"
#include "vectorType.H"


VectorType::VectorType(int _size) :
  size(_size), values(new float[_size]), calculated(0) {
    for (int i = 0 ; i < size ; i++)
      values[i] = 0;
}

VectorType::VectorType(const VectorType& _vec) {
  calculated = _vec.calculated;
  size = _vec.size;
  values = new float [size];
  for (int i = 0 ; i < size ; i++) {
    values[i] = _vec.values[i];
  }
}

VectorType::VectorType(float val1, float val2) :
  size(2), values(new float[2])
{
  values[0] = val1;
  values[1] = val2;
}


VectorType&
VectorType::operator= (const VectorType& _vec) {
  //    if (this == _vec.this) return *this;
  if (size != _vec.size) {
    delete[] values;
    values = new float [size];
  }
  calculated = _vec.calculated;
  for (int i = 0 ; i < size ; i++) {
    values[i] = _vec.values[i];
  }
  return *this;
}


float
VectorType::distance_L1 (const VectorType& v, float threshold) {
  float dist = 0;
  int count =0;
  for (int i=0 ; i<size ; i++)
    if ( (values[i]>=threshold) || (v.values[i]>=threshold) ) {
      count++;
      dist += (float) fabs(values[i] - v.values[i]);
    }
  return dist;
}


float
VectorType::distance_L2 (const VectorType& v, float threshold) {
  float dist = 0, diff =0;
  int count =0;
  for (int i=0 ; i<size ; i++)
    if ( (values[i]>=threshold) || (v.values[i]>=threshold) ) {
      diff = values[i] - v.values[i];
      dist += (diff * diff);
    }
  return sqrt(dist);
}


float
VectorType::distance_CCV (const VectorType& v, float threshold) {
  // check equal size first; otherwise reject
  if (size != v.size)
    return FLT_MAX;
  cout << "THRESHOLD " << threshold << endl;
  double dist = 0.0;
  int count =0;
  for (int i=0 ; i<size ; i++) {
    if ( (values[i]>=threshold) || (v.values[i]>=threshold) ) {
      count++;
      dist += fabs ((double)(values[i] - v.values[i]) /
		    (double)(values[i] + v.values[i]));
    }
  }
  return (dist/double(count));
}


void
VectorType::write(FILE *file) {
  fwrite (&calculated, sizeof(unsigned char), 1, file);
  // fwrite (values, sizeof(float), size, file);
  for (int i=0; i<size ; i++) {
    float value=values[i];
#ifdef DAMN_INTEL_BYTE_ORDER
    swap_int ((char*)&value);
#endif
    fwrite(&value, sizeof(float), 1, file);
  }
}

void
VectorType::read(FILE *file) {
  fread(&calculated, sizeof(unsigned char), 1, file);
  // fread(values, sizeof(float), size, file);
  for (int i=0; i<size ; i++) {
    float value;
    fread(&value, sizeof(float), 1, file);
#ifdef DAMN_INTEL_BYTE_ORDER
    swap_int ((char*)&value);
#endif
    values[i]=value;
  }
}


float
VectorType::max()
{
  float max = FLT_MIN;
  for (int i=0 ; i<size ; i++)
    if (max < values[i])
      max = values[i];
  return max;
}


float
VectorType::min()
{
  float min = FLT_MAX;
  for (int i=0 ; i<size ; i++)
    if (min > values[i])
      min = values[i];
  return min;
}


ostream&
operator<<(ostream& os, const VectorType& v) {
  // print alpha values, divided by spaces
  for (long i = 0; i < v.size; i++)
    os << v.values[i] << " ";
  os << endl;
  return os;
}
