/*
   This file is part of the Aulib library.
   Name: zerocrossing.h
   Copyright (C) 2004 Silvia Pfeiffer

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: pfeiffer@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

#ifndef _AU_ZEROCROSSING_H_
#define _AU_ZEROCROSSING_H_

#include "autypes.h"

#include <cmath>

using namespace std;

class AudioCache;

/**
 * This class implements routines to calculate zero crossing rate
 * in the time domain.
 *
 * @author <a href = "mailto:pfeiffer@pi4.informatik.uni-mannheim.de">Silvia Pfeiffer</a>
 * @version $Id: zerocrossing.h,v 1.2 1998/06/04 09:30:58 pfeiffer Exp $
 */
class AuZerocrossing {

  public:
    /**
      * Function to calculate zero crossing rate on the samples stored in the
      * audio cache. The original definition of the zero crossing rate of a
      * discrete-time signal is:<BR>
      * z = lim{n->infty} 1/sampling_rate *
      *     sum{-n to n}( {|sign(x(n)-sign(x(n+1))|} / 2 )
      *
      * @param cache audio cache containing the signal
      * @return      calculated zero crossing rate
      */
    static double zerocrossing(const AudioCache& cache);

};

#endif

